
clear all
close all

num_bits = 63; %number of bits in the MLS to use
logbase2pow = ceil(log2(num_bits));%get base power of 2 for number of bits desired

MLS_seq = mseq(2,logbase2pow,0,0); %generates an MLS 
MLS_seq = MLS_seq(1:num_bits); %truncates only the first bits of the MLS desired

%Stretch the MLS such that each bit = 10 points, or that each bit = 10
%pixels in in the CMOS frame
x = 1:length(MLS_seq);
v = MLS_seq;
xq = 1:.1:length(MLS_seq);
vq2 = 10*(interp1(x,v,xq,'nearest')+1);


%Store a signal matrix 
Signal_Mat = zeros(27,1920);

%Create several signals that are spaced apart
Signal_Mat(1,1:length(vq2)) = vq2;
for i = 2:27
  Signal_Mat(i,1+floor((i-1)*length(vq2)/13):length(vq2)+floor((i-1)*length(vq2)/13)) = vq2;
end

%Add white noise to the signal
snr = -10;
for i = 1:27
    Signal_Mat(i,:) = awgn(Signal_Mat(i,:),snr);
end

%Plot the first, middle, and last pattern in the matrix
figure
stairs(Signal_Mat(1,:),'Color',[1 0 0])
hold on
stairs(Signal_Mat(13,:),'Color',[0 1 0])
hold on
stairs(Signal_Mat(27,:),'Color',[0 0 1])

Signal = zeros(1,length(Signal_Mat(1,:)));

%Add two signals that are next to eaceh other
for i = 13:14
    Signal = Signal + Signal_Mat(i,:);
end

%Find correlatoin between the expected and summed signals
corr_vector = xcorr(vq2,Signal);
figure
subplot(2,1,1)
plot(Signal)
title('Summed Signal of 2 64-bit patterns spaced 23 px apart')
subplot(2,1,2)
plot(corr_vector)
title('Correlation of expected signal with Summed Signal')


figure
stairs(Signal,'LineWidth',1)
title('Summed signal')
axis([0 1920 0 300])

%Filter out DC and low frequencies from the correlation 
figure
Fs = 1920;
half_freq = Fs/2;
notch_freq_filter =1;
low_freq_filt = 2;
smoothing_factor = 2;
w0=notch_freq_filter/half_freq;
[num,den]=iirnotch(w0,w0/35,-45);
[a,b]=butter(2,low_freq_filt/half_freq,'high');
 
y=filter(num,den,corr_vector);
yfil=filtfilt(a,b,y); 
ysm=smooth(yfil,smoothing_factor);  
figure
plot(abs(ysm))

figure

%Due to the shift in the correlation, only a segment of it is relevant to
%when the expected and signal of interest overlap
actual_vector_that_matters = abs(ysm(640:1920+640));
actual_vector_that_matters=actual_vector_that_matters/max(actual_vector_that_matters);
plot(actual_vector_that_matters,'LineWidth',2)
%select a threshold
threshold2 = max(actual_vector_that_matters)*.4;
%Find peaks based on a minimal peak distance
[pks,locs] = findpeaks(actual_vector_that_matters,'MinPeakDistance',20,'MinPeakHeight',threshold2);

%Plot the information
threshold_line_x = 1:20:length(actual_vector_that_matters);
threshold_line_y = zeros(1,length(threshold_line_x))+threshold2;

hold on
plot(locs,pks,'ro','MarkerSize',6,'LineWidth',2)
hold on
plot(threshold_line_x,threshold_line_y,':','Color',[1 0 0],'LineWidth',4)
axis([0 1920 0 1.2])
xlabel('Frame Pixel')
ylabel('Signal Correlation')
title('Max beads that can be packed into a frame')


%Plot all the relevant information onto one figure
figure
subplot(4,1,1)
stairs(Signal_Mat(13,:),'LineWidth',1)
axis([400 1400 0 50])
title('Signal 1')
grid off 
box off
subplot(4,1,2)
title('Signal 2')
stairs(Signal_Mat(14,:),'LineWidth',1)
axis([400 1400 0 50])
grid off 
box off
subplot(4,1,3)
stairs(Signal_Mat(14,:)+Signal_Mat(13,:),'LineWidth',1)
title('Signals Overlap')
axis([400 1400 0 50])
grid off 
box off
subplot(4,1,4)
plot(actual_vector_that_matters,'LineWidth',2)
hold on
plot(locs,pks,'ro','MarkerSize',6,'LineWidth',2)
title('Correlation peaks')
plot(threshold_line_x,threshold_line_y,':','Color',[1 0 0],'LineWidth',4)
axis([400 1400  0 1.2])

grid off 
box off
%   set(gca,'visible','off');

