%detect peaks in array
%searches through the passed 2d matrix for a given segmented channel if droplets appear more than 2+ frame in series
%based on expected velocity to designate as a true positive

%initialize variables and input data
hough_array = corr_matrix_per_channel';
[hough_array_a, hough_array_b] = size(hough_array);
loc_info_all = zeros(hough_array_b+3,20);
full_matrix_pks_locs = zeros(hough_array_b,100,100);


for i = 1:hough_array_b
    %find peaks in the data based on the threshold set and the expected
    %minimum distance between droplets
[pks,locs] = findpeaks(hough_array(:,i),'MinPeakDistance',20,'MinPeakHeight',threshold2);
%store the pk values and the locations in a matrix
full_matrix_pks_locs(i,1:length(locs),1:2) = [pks locs];
loc_info_all(i,1:length(locs)) = locs;
end

%find the total number of peaks that were found in the matrix by
%counting all entires that are nonzero as peaks that passed the
%thresholding step
num_totalpeaks_initial = nnz(loc_info_all);

%we now loop through and find repeating peaks. 
%if peaks are within the expected velocity range, and occur at least twice
%we count them as real droplets. If a peak only occurs once with no
%followup, this can be attributed to noise. 
expected_velocity = 540;
range = 50;
for i = 1:hough_array_b
    set_of_locs = loc_info_all(i,:);
    for j = 1:length(set_of_locs)
        if set_of_locs(j) ~=0
            repeatcount = 0; %this variable checks if a droplet was able to be seen two or three times successfuly.
            locationvalue = set_of_locs(j);
             nextlocationvalue = locationvalue+expected_velocity; %this finds the next location value based on the ranges selected
             set_of_locs2 = loc_info_all(i+1,:); 
             for k=1:length(set_of_locs2)
                 if set_of_locs2(k)>nextlocationvalue-range && set_of_locs2(k)<nextlocationvalue+range
                 repeatcount = 1; %if a droplet is within the range, this means a peak was successfully found in the given range of prediction
                 if repeatcount==1 
                     nextlocationvalue2 = nextlocationvalue +expected_velocity; %look for a third peak 
                      set_of_locs3 = loc_info_all(i+2,:);
                                  for l=1:length(set_of_locs2)
                                     if set_of_locs3(l)>nextlocationvalue2-range && set_of_locs3(l)<nextlocationvalue2+range
                                     repeatcount = 2; 
                                      if repeatcount==2
                                          loc_info_all(i+2,l) = 0; break %set repeat peaks to 0 so droples are not overcounted (three peaks case)
                                      end
                                     end
                                  end
                     loc_info_all(i+1,k) = 0; %set repeat peaks to 0 so droples are not overcounted (two peaks case)
                     break
                     %in cases of repeat peaks, the original location still
                     %remains, so this is the peak that gets counted as a
                     %true positive                     
                 end  
                 end   
             end
             if repeatcount==0
                 loc_info_all(i,j) = 0; %if a droplet was only detected one time, then discard it. This peak corresponds to a false positive
             end
        end
    end  
end

%find all the remaining peaks that correspond to true droplets rather than
%repeat peaks or single peaks from noise. 
num_totalpeaks_final = nnz(loc_info_all);
nonzeros_per_channel= sum(loc_info_all~=0,2);