%initializes all variables such as expected signals, camera parameters, etc. 
%sets the frames to view within the video to start analysis
%calls Frame_Undistrotion to start a chain to .m files that creates a matrix where correlations are stored
%once the vector is saved into a .mat file that can also be accessed later as done typically...
%calls Corr_Hough_matrix_script to begin a chain for peak finding analysis


close all force  
clear all
% parpool
%uncommeting the above can start the parallel computing toolbox if
%available in matlab. Doing so increases computational speed significantly

%initialize variables
x1 = 0 ;
x2 = 0;
y1 = 0;
y2 = 0;
inc = 0;

%load the MLS that was used in the Arduino Code
 MLS_seq = [-1, 1, -1,  1, -1,  1, 1, -1,  -1,  1, 1, -1,  1, 1, 1, -1,  1, 1, -1,  1, -1,  -1,  1, -1,  -1 , 1 ,1 ,1, -1,  -1,  -1,  1, -1,  1, 1, 1, 1, -1,  -1,  1, -1,  1, -1,  -1,  -1,  1 ,1, -1,  -1,  -1,  -1,  1, -1,  -1 , -1,  -1,  -1,  1, 1, 1, 1, 1, 1];
%load camera params once to fix image distortion
%The main variables that affect spherical distortion is 'RadialDistortion'
load('cameraParams1.mat')
cameraParams2 = cameraParameters('IntrinsicMatrix', [716,0,0;0,716,0;1920/2,1080/2,1],...
    'RadialDistortion',[.06,0.0,.00],...
    'TangentialDistortion',[0,0],...
    'RotationVectors',[0 0 0],...
    'TranslationVectors',[0 0 0],...
    'ReprojectionErrors',[0 0],...
    'WorldPoints',cameraParams1.WorldPoints,...
    'WorldUnits',cameraParams1.WorldUnits,...
    'EstimateSkew',cameraParams1.EstimateSkew,...
    'NumRadialDistortionCoefficients',cameraParams1.NumRadialDistortionCoefficients,...
    'EstimateTangentialDistortion',cameraParams1.EstimateTangentialDistortion);


%select amount of channels and segment based on the pixels per channel
segmented_ch_num = 120;
pixels_per_ch = floor(1080/segmented_ch_num);

%load the interpolated masks. these represent the expected signal
%NOTE: the interpolated masks can change to fit the velocity range required
%and can be increased or shortened to improve computation. In this
%particular case, the range was chosen to match the data, and obviously,
%larger ranges with finer resolution will always work but the tradeoff is
%computation time. 
load('All_Interpolated_Masks.mat')

%the table below has several values, but the main ones that are relevant
%are the first entry which is the filename of a video to be analyzed, and
%the last term (flip) which represents the direction the droplets are going
%in. 

%Droplets can either go left to right or right to left based on how the
%chpi is attached; the flip vector properly selects the option

%The remaining variables can be ignored for this code.
data_table = {'20160927_165111.mp4'	27	7	624	800	1142	800	0	624	392	1142	377	624	1142	1};
load('cmap2.mat')

filename = data_table{1}; frame = data_table{2}; 
flip = data_table{15}; inc = data_table{8};
x1 = data_table{4}; y1 =data_table{5}-inc; x2 = data_table{6}; y2 = data_table{7} -inc;
%bead two start and end
x3 = data_table{9}; y3 = data_table{10}; x4 = data_table{11}; y4 = data_table{12};
dropletstart = data_table{13}; dropletend = data_table{14};


startframe = 450; endframe = 650; %select frames in video to analyze
%a portion of the video is analyzed since there may be a lag time for
%droplets to reach the proper velocity initially, saving computation time.

%select the background frame to subtract out background signal in the
%algorithm
background_frame = 1;
v_file = VideoReader(filename);
video_frame = read(v_file,background_frame);
R_channel = video_frame(:,:,1);
R_background = R_channel;

%begin loop through each frame
for framestoprocess = 1:endframe-startframe+1
    framenum=framestoprocess ;
    frame = startframe+framestoprocess-1;
    fprintf(num2str(framestoprocess))
    %%%%%%%%%%%%%%%%%%%%%% Workflow Chain Begins %%%%%%%%%%%%%%%%%%%%%%%%%%
    Frame_Undistrotion

end

%save the correlation results in a matrix for analysis later if necessary
save('Frames450to650.mat','-v7.3','corr_matrix_final')
%begin analysis on the correlation matrix
Corr_Hough_matrix_script